from utilib.core import http
import json
import hashlib


async def get_access_token(appid, secret):
    _r = await http.async_fetch_url(
        'https://api.weixin.qq.com/cgi-bin/token',
        'GET',
        {},
        True,
        grant_type='client_credential',
        appid=appid,
        secret=secret
    )
    return json.loads(_r.body.decode())


async def get_jsapi_ticket(access_token):
    _r = await http.async_fetch_url(
        'https://api.weixin.qq.com/cgi-bin/ticket/getticket',
        'GET',
        {},
        True,
        type='jsapi',
        access_token=access_token
    )
    return json.loads(_r.body.decode())


async def get_oauth_access_token(appid, secret, code):
    _r = await http.async_fetch_url(
        'https://api.weixin.qq.com/sns/oauth2/access_token',
        'GET',
        {},
        True,
        appid=appid,
        secret=secret,
        code=code,
        grant_type='authorization_code'
    )
    return json.loads(_r.body.decode())


async def get_user_info(access_token, openid):
    _r = await http.async_fetch_url(
        'https://api.weixin.qq.com/sns/userinfo',
        'GET',
        {},
        True,
        access_token=access_token,
        openid=openid,
        lang='zh_CN'
    )
    return json.loads(_r.body.decode())


def get_signature(**kwargs):
    return hashlib.sha1(
        ('&'.join(
            str.format(
                '{k}={v}', k=k, v=v) for k, v in sorted(
                kwargs.items()))).encode('utf-8')).hexdigest()




CODE_0_SUCCESS = 0
CODE_1_ERROR = 1
CODE_2_LOGIN_REQUIRED = 2
CODE_3_ERROR_SECURE_TOKEN = 3
CODE_4_ERROR_USERNAME_OR_PASSWORD = 4
CODE_5_INVALID_PASSWORD = 5
CODE_6_USER_CREATION_FAILED = 6
CODE_7_SEND_SMS_FAILED = 7
CODE_8_CAPTCHA_VALIDATION_ERROR = 8
CODE_9_INVALID_PHONE_NUMBER = 9
CODE_10_INVALID_EMAIL_ADDRESS = 10
CODE_11_INVALID_USERNAME = 11
CODE_12_INVALID_CHINESE_NAME = 12
CODE_13_VCODE_VALIDATION_ERROR = 13
CODE_14_PHONE_ALREADY_EXISTS_ERROR = 14
CODE_15_USERNAME_ALREADY_EXISTS_ERROR = 15
CODE_16_EMAIL_ALREADY_EXISTS_ERROR = 16
CODE_17_SALESMAN_ROLE_REQUIRED = 17
CODE_18_ADVERTISER_REQUIRED = 18
CODE_19_MISSING_REQUIRED_PARAMS = 19
CODE_20_PERMISSION_DENIED = 20
CODE_21_INVALID_PARAMETER = 21
CODE_22_SHOPKEEPER_REQUIRED = 22
CODE_23_OBJECT_ALREADY_EXISTS = 23


class APIException(Exception):

    code = CODE_1_ERROR
    default_detail = 'A server error occurred.'
    default_detail_zh = '服务器发生了一个错误'

    def __init__(self, detail=None, chinese_detail=True):
        if detail is not None:
            self.detail = detail
        else:
            self.detail = self.default_detail
            if chinese_detail:
                self.detail = self.default_detail_zh

    def __str__(self):
        return self.detail


class LoginRequired(APIException):
    code = CODE_2_LOGIN_REQUIRED
    default_detail = 'Login required.'

    default_detail_zh = '请登陆后再进行此操作'


class SecureTokenError(APIException):

    code = CODE_3_ERROR_SECURE_TOKEN
    default_detail = 'Error secure token.'


class UsernameOrPasswordError(APIException):

    code = CODE_4_ERROR_USERNAME_OR_PASSWORD
    default_detail = 'Error username or password.'

    default_detail_zh = '错误的用户名或密码'


class InvalidPassword(APIException):

    code = CODE_5_INVALID_PASSWORD
    default_detail = 'Invalid password'

    default_detail_zh = '密码不符合要求'


class UserCreationError(APIException):

    code = CODE_6_USER_CREATION_FAILED
    default_detail = 'UserCreationError.'

    default_detail_zh = '创建用户失败'


class SMSSendFailed(APIException):

    code = CODE_7_SEND_SMS_FAILED
    default_detail = 'Send SMS failed'

    default_detail_zh = '短信发送失败'


class CaptchaValidationError(APIException):

    code = CODE_8_CAPTCHA_VALIDATION_ERROR
    default_detail = 'Captcha validation error'

    default_detail_zh = '验证码不匹配'


class InvalidPhoneNumber(APIException):

    code = CODE_9_INVALID_PHONE_NUMBER
    default_detail = 'Invalid phone number'

    default_detail_zh = '不正确的手机号码'


class InvalidEmailAddress(APIException):

    code = CODE_10_INVALID_EMAIL_ADDRESS
    default_detail = 'Invalid email address'

    default_detail_zh = '不正确的邮件地址'


class InvalidUsername(APIException):

    code = CODE_11_INVALID_USERNAME
    default_detail = 'Invalid username'

    default_detail_zh = '不合法的用户名'


class InvalidChineseName(APIException):

    code = CODE_12_INVALID_CHINESE_NAME
    default_detail = 'Invalid Chinese name'

    default_detail_zh = '不正确的中文姓名'


class VcodeValidationError(APIException):

    code = CODE_13_VCODE_VALIDATION_ERROR
    default_detail = 'Vcode validation error'

    default_detail_zh = '验证码不正确或已经失效'


class PhoneAlreadyExistsError(APIException):

    code = CODE_14_PHONE_ALREADY_EXISTS_ERROR
    default_detail = 'Phone already exists'

    default_detail_zh = '手机号已经被注册'


class UsernameAlreadyExistsError(APIException):

    code = CODE_15_USERNAME_ALREADY_EXISTS_ERROR
    default_detail = 'Username already exists'

    default_detail_zh = '用户名已存在'


class EmailAlreadyExistsError(APIException):

    code = CODE_16_EMAIL_ALREADY_EXISTS_ERROR
    default_detail = 'Email already exists'

    default_detail_zh = '邮箱地址已经被注册'


class SalesmanRequired(APIException):

    code = CODE_17_SALESMAN_ROLE_REQUIRED
    default_detail = 'A salesman account is required'

    default_detail_zh = '需要以地推人员账号登录'


class AdvertiserRequired(APIException):

    code = CODE_18_ADVERTISER_REQUIRED
    default_detail = 'An advertiser account is required'

    default_detail_zh = '需要以广告主账号登录'


class MissingRequiredParams(APIException):

    code = CODE_19_MISSING_REQUIRED_PARAMS
    default_detail = 'Required params is missing'

    default_detail_zh = '缺少必要参数'


class PermissionDenied(APIException):

    code = CODE_20_PERMISSION_DENIED
    default_detail = 'Permission denied'

    default_detail_zh = '没有权限查看此内容'


class InvalidParameterValue(APIException):

    code = CODE_21_INVALID_PARAMETER
    default_detail = 'Invalid parameter value'

    default_detail_zh = '不正确的参数值'


class ShopkeeperRequired(APIException):

    code = CODE_22_SHOPKEEPER_REQUIRED
    default_detail = 'An shopkeeper account is required'

    default_detail_zh = '需要以店主账号登录'


class ObjectAlreadyExists(APIException):

    code = CODE_23_OBJECT_ALREADY_EXISTS
    default_detail = 'Object already exists'

    default_detail_zh = '此项已经存在'



color_names = (
    'black',
    'red',
    'green',
    'yellow',
    'blue',
    'magenta',
    'cyan',
    'white')
foreground = {color_names[x]: '3%s' % x for x in range(8)}
background = {color_names[x]: '4%s' % x for x in range(8)}

RESET = '0'
opt_dict = {
        'bold': '1',
        'underscore': '4',
        'blink': '5',
        'reverse': '7',
        'conceal': '8'
    }


def colorize(text='', opts=(), **kwargs):
    """
    Return your text, enclosed in ANSI graphics codes.
    Depends on the keyword arguments 'fg' and 'bg', and the contents of
    the opts tuple/list.
    Return the RESET code if no parameters are given.
    Valid colors:
        'black', 'red', 'green', 'yellow', 'blue', 'magenta', 'cyan', 'white'
    Valid options:
        'bold'
        'underscore'
        'blink'
        'reverse'
        'conceal'
        'noreset' - string will not be auto-terminated with the RESET code
    Examples:
        colorize('hello', fg='red', bg='blue', opts=('blink',))
        colorize()
        colorize('goodbye', opts=('underscore',))
        print(colorize('first line', fg='red', opts=('noreset',)))
        print('this should be red too')
        print(colorize('and so should this'))
        print('this should not be red')
    """
    code_list = []
    if text == '' and len(opts) == 1 and opts[0] == 'reset':
        return '\x1b[%sm' % RESET
    for k, v in kwargs.items():
        if k == 'fg':
            code_list.append(foreground[v])
        elif k == 'bg':
            code_list.append(background[v])
    for o in opts:
        if o in opt_dict:
            code_list.append(opt_dict[o])
    if 'noreset' not in opts:
        text = '%s\x1b[%sm' % (text or '', RESET)
    return '%s%s' % (('\x1b[%sm' % ';'.join(code_list)), text or '')


def make_style(opts=(), **kwargs):
    """
    Return a function with default parameters for colorize()
    Example:
        bold_red = make_style(opts=('bold',), fg='red')
        print(bold_red('hello'))
        KEYWORD = make_style(fg='yellow')
        COMMENT = make_style(fg='blue', opts=('bold',))
    """
    return lambda text: colorize(text, opts, **kwargs)


NOCOLOR_PALETTE = 'nocolor'
DARK_PALETTE = 'dark'
LIGHT_PALETTE = 'light'

PALETTES = {
    NOCOLOR_PALETTE: {
        'ERROR': {},
        'SUCCESS': {},
        'WARNING': {},
        'NOTICE': {},
        'SQL_FIELD': {},
        'SQL_COLTYPE': {},
        'SQL_KEYWORD': {},
        'SQL_TABLE': {},
        'HTTP_INFO': {},
        'HTTP_SUCCESS': {},
        'HTTP_REDIRECT': {},
        'HTTP_NOT_MODIFIED': {},
        'HTTP_BAD_REQUEST': {},
        'HTTP_NOT_FOUND': {},
        'HTTP_SERVER_ERROR': {},
        'MIGRATE_HEADING': {},
        'MIGRATE_LABEL': {},
    },
    DARK_PALETTE: {
        'ERROR': {'fg': 'red', 'opts': ('bold',)},
        'SUCCESS': {'fg': 'green', 'opts': ('bold',)},
        'WARNING': {'fg': 'yellow', 'opts': ('bold',)},
        'NOTICE': {'fg': 'red'},
        'SQL_FIELD': {'fg': 'green', 'opts': ('bold',)},
        'SQL_COLTYPE': {'fg': 'green'},
        'SQL_KEYWORD': {'fg': 'yellow'},
        'SQL_TABLE': {'opts': ('bold',)},
        'HTTP_INFO': {'opts': ('bold',)},
        'HTTP_SUCCESS': {},
        'HTTP_REDIRECT': {'fg': 'green'},
        'HTTP_NOT_MODIFIED': {'fg': 'cyan'},
        'HTTP_BAD_REQUEST': {'fg': 'red', 'opts': ('bold',)},
        'HTTP_NOT_FOUND': {'fg': 'yellow'},
        'HTTP_SERVER_ERROR': {'fg': 'magenta', 'opts': ('bold',)},
        'MIGRATE_HEADING': {'fg': 'cyan', 'opts': ('bold',)},
        'MIGRATE_LABEL': {'opts': ('bold',)},
    },
    LIGHT_PALETTE: {
        'ERROR': {'fg': 'red', 'opts': ('bold',)},
        'SUCCESS': {'fg': 'green', 'opts': ('bold',)},
        'WARNING': {'fg': 'yellow', 'opts': ('bold',)},
        'NOTICE': {'fg': 'red'},
        'SQL_FIELD': {'fg': 'green', 'opts': ('bold',)},
        'SQL_COLTYPE': {'fg': 'green'},
        'SQL_KEYWORD': {'fg': 'blue'},
        'SQL_TABLE': {'opts': ('bold',)},
        'HTTP_INFO': {'opts': ('bold',)},
        'HTTP_SUCCESS': {},
        'HTTP_REDIRECT': {'fg': 'green', 'opts': ('bold',)},
        'HTTP_NOT_MODIFIED': {'fg': 'green'},
        'HTTP_BAD_REQUEST': {'fg': 'red', 'opts': ('bold',)},
        'HTTP_NOT_FOUND': {'fg': 'red'},
        'HTTP_SERVER_ERROR': {'fg': 'magenta', 'opts': ('bold',)},
        'MIGRATE_HEADING': {'fg': 'cyan', 'opts': ('bold',)},
        'MIGRATE_LABEL': {'opts': ('bold',)},
    }
}
DEFAULT_PALETTE = DARK_PALETTE


def parse_color_setting(config_string):
    """Parse a DJANGO_COLORS environment variable to produce the system palette
    The general form of a palette definition is:
        "palette;role=fg;role=fg/bg;role=fg,option,option;role=fg/bg,option,option"
    where:
        palette is a named palette; one of 'light', 'dark', or 'nocolor'.
        role is a named style used by Django
        fg is a background color.
        bg is a background color.
        option is a display options.
    Specifying a named palette is the same as manually specifying the
    individual
    definitions for each role. Any individual definitions following the palette
    definition will augment the base palette definition.
    Valid roles:
        'error', 'success', 'warning', 'notice', 'sql_field', 'sql_coltype',
        'sql_keyword', 'sql_table', 'http_info', 'http_success',
        'http_redirect', 'http_not_modified', 'http_bad_request',
        'http_not_found', 'http_server_error', 'migrate_heading',
        'migrate_label'
    Valid colors:
        'black', 'red', 'green', 'yellow', 'blue', 'magenta', 'cyan', 'white'
    Valid options:
        'bold', 'underscore', 'blink', 'reverse', 'conceal', 'noreset'
    """
    if not config_string:
        return PALETTES[DEFAULT_PALETTE]

    # Split the color configuration into parts
    parts = config_string.lower().split(';')
    palette = PALETTES[NOCOLOR_PALETTE].copy()
    for part in parts:
        if part in PALETTES:
            # A default palette has been specified
            palette.update(PALETTES[part])
        elif '=' in part:
            # Process a palette defining string
            definition = {}

            # Break the definition into the role,
            # plus the list of specific instructions.
            # The role must be in upper case
            role, instructions = part.split('=')
            role = role.upper()

            styles = instructions.split(',')
            styles.reverse()

            # The first instruction can contain a slash
            # to break apart fg/bg.
            colors = styles.pop().split('/')
            colors.reverse()
            fg = colors.pop()
            if fg in color_names:
                definition['fg'] = fg
            if colors and colors[-1] in color_names:
                definition['bg'] = colors[-1]

            # All remaining instructions are options
            opts = tuple(s for s in styles if s in opt_dict)
            if opts:
                definition['opts'] = opts

            # The nocolor palette has all available roles.
            # Use that palette as the basis for determining
            # if the role is valid.
            if role in PALETTES[NOCOLOR_PALETTE] and definition:
                palette[role] = definition

    # If there are no colors specified, return the empty palette.
    if palette == PALETTES[NOCOLOR_PALETTE]:
        return None
    return palette



from utilib.core import exceptions
import re
import string
import random


def is_phone(_str, raise_exception=False):
    """
    Determine if a string param is a phone number.
    """
    if re.match(r'^(\+86|)1[0-9]{10}$', _str):
        return True
    if raise_exception:
        raise exceptions.InvalidPhoneNumber
    return False


def is_email(_str, raise_exception=False):
    if re.match(
        r'^[0-9a-zA-Z_]+@[0-9a-zA-Z\-\u4e00-\u9fa5]'
        r'+\.(com|cn|com\.cn|edu\.cn|net)$',
            _str):
        return True
    if raise_exception:
        raise exceptions.InvalidEmailAddress
    return False


def is_username(_str, raise_exception=False):
    if re.match(r'^[A-Za-z][0-9a-zA-Z_]{5,15}$', _str):
        return True
    if raise_exception:
        raise exceptions.InvalidUsername
    return False


def is_chinese_name(_str, raise_exception=False):
    if re.match(r'^[\u4e00-\u9fa5]{2,}$', _str):
        return True
    if raise_exception:
        raise exceptions.InvalidChineseName
    return False


def is_password(_str, raise_exception=False):
    if re.match(r'^(?=.*[A-Za-z])(?=.*\d)[A-Za-z\d$@$!%*#?&.]{8,}$', _str):
        return True
    if raise_exception:
        raise exceptions.InvalidPassword
    return False


def gen_sms_vcode(size=6):
    return ''.join(random.choice(string.digits) for _ in range(size))


def gen_captcha(size=6):
    return ''.join(random.choice(string.ascii_letters +
                                 string.digits) for _ in range(size))


def gen_nonce_str(size=16):
    return ''.join(random.choice(string.ascii_letters +
                                 string.digits) for _ in range(size))


def to_str(_str):
    if isinstance(_str, bytes):
        return _str.decode()
    return str(_str)



from io import BytesIO
import pyexcel


def make_excel(record_list,
               file_type='xlsx', sheet_name='sheet1', headers=[]):
    record_list.insert(0, headers)
    _out = BytesIO()
    pyexcel.Sheet(record_list).save_to_memory(file_type, _out)
    return _out



from tornado.httpclient import AsyncHTTPClient, HTTPRequest, HTTPError
from utilib.core import exceptions
import urllib


class VcodeHelper:

    def __init__(self, apikey):
        self.apikey = apikey

    async def send_sms_vcode(self, code, mobile, expire=2):
        template = '【超盟数据】您的验证码是{code}，有效期为{minute}分钟。如非本人操作，请忽略本短信'
        _data = {
            'apikey': self.apikey,
            'mobile': mobile,
            'text': str.format(template, code=code, minute=expire)
        }
        request = HTTPRequest(
            'https://sms.yunpian.com/v2/sms/single_send.json',
            body=urllib.parse.urlencode(_data), method='POST')
        http_client = AsyncHTTPClient()
        try:
            await http_client.fetch(request)
        except HTTPError:
            raise exceptions.SMSSendFailed

    async def send_voice_vcode(self, code, mobile):
        _data = {
            'apikey': self.apikey,
            'mobile': mobile,
            'code': code
        }
        request = HTTPRequest(
            'https://voice.yunpian.com/v2/voice/send.json',
            body=urllib.parse.urlencode(_data), method='POST')
        http_client = AsyncHTTPClient()
        await http_client.fetch(request)


def require_params(*args):
    if None in args:
        raise exceptions.MissingRequiredParams



from PIL import Image
from PIL import ImageFilter
from PIL.ImageDraw import Draw
from PIL.ImageFont import truetype
from io import BytesIO
import random
import pkgutil

DEFAULT_FONTS = pkgutil.get_data('fonts')

table = []
for i in range(256):
    table.append(i * 1.97)


class _Captcha(object):

    def generate(self, chars, format='png'):
        """Generate an Image Captcha of the given characters.
        :param chars: text to be generated.
        :param format: image file format
        """
        im = self.generate_image(chars)
        out = BytesIO()
        im.save(out, format=format)
        out.seek(0)
        return out

    def write(self, chars, output, format='png'):
        """Generate and write an image CAPTCHA data to the output.
        :param chars: text to be generated.
        :param output: output destionation.
        :param format: image file format
        """
        im = self.generate_image(chars)
        return im.save(output, format=format)


class ImageCaptcha(_Captcha):

    def __init__(self, width=160, height=80, fonts=None, font_sizes=None):
        self._width = width
        self._height = height
        self._fonts = fonts or DEFAULT_FONTS
        self._font_sizes = font_sizes or (46, 58, 68)
        self._truefonts = []

    @property
    def truefonts(self):
        if self._truefonts:
            return self._truefonts
        self._truefonts = tuple([
            truetype(n, s)
            for n in self._fonts
            for s in self._font_sizes
        ])
        return self._truefonts

    @staticmethod
    def create_noise_curve(image, color):
        w, h = image.size
        x1 = random.randint(0, int(w / 5))
        x2 = random.randint(w - int(w / 5), w)
        y1 = random.randint(h / 5, h - int(h / 5))
        y2 = random.randint(y1, h - int(h / 5))
        points = [(x1, y1), (x2, y2)]
        end = random.randint(160, 200)
        start = random.randint(0, 20)
        Draw(image).arc(points, start, end, fill=color)
        return image

    @staticmethod
    def create_noise_dots(image, color, width=3, number=30):
        draw = Draw(image)
        w, h = image.size
        while number:
            x1 = random.randint(0, w)
            y1 = random.randint(0, h)
            draw.line(((x1, y1), (x1 - 1, y1 - 1)), fill=color, width=width)
            number -= 1
        return image

    def create_captcha_image(self, chars, color, background):

        image = Image.new('RGB', (self._width, self._height), background)
        draw = Draw(image)

        def _draw_character(c):
            font = random.choice(self.truefonts)
            w, h = draw.textsize(c, font=font)

            dx = random.randint(0, 4)
            dy = random.randint(0, 6)
            im = Image.new('RGBA', (w + dx, h + dy))
            Draw(im).text((dx, dy), c, font=font, fill=color)

            # rotate
            im = im.crop(im.getbbox())
            im = im.rotate(random.uniform(-30, 30), Image.BILINEAR, expand=1)

            # warp
            dx = w * random.uniform(0.1, 0.3)
            dy = h * random.uniform(0.2, 0.3)
            x1 = int(random.uniform(-dx, dx))
            y1 = int(random.uniform(-dy, dy))
            x2 = int(random.uniform(-dx, dx))
            y2 = int(random.uniform(-dy, dy))
            w2 = w + abs(x1) + abs(x2)
            h2 = h + abs(y1) + abs(y2)
            data = (
                x1, y1,
                -x1, h2 - y2,
                w2 + x2, h2 + y2,
                w2 - x2, -y1,
            )
            im = im.resize((w2, h2))
            im = im.transform((w, h), Image.QUAD, data)
            return im

        images = []
        for c in chars:
            images.append(_draw_character(c))

        text_width = sum([im.size[0] for im in images])

        width = max(text_width, self._width)
        image = image.resize((width, self._height))

        average = int(text_width / len(chars))
        rand = int(0.25 * average)
        offset = int(average * 0.1)

        for im in images:
            w, h = im.size
            mask = im.convert('L').point(table)
            image.paste(im, (offset, int((self._height - h) / 2)), mask)
            offset = offset + w + random.randint(-rand, 0)

        return image

    def generate_image(self, chars):
        """Generate the image of the given characters.
        :param chars: text to be generated.
        """
        background = random_color(238, 255)
        color = random_color(0, 200, random.randint(220, 255))
        im = self.create_captcha_image(chars, color, background)
        self.create_noise_dots(im, color)
        self.create_noise_curve(im, color)
        im = im.filter(ImageFilter.SMOOTH)
        return im


def random_color(start, end, opacity=None):
    red = random.randint(start, end)
    green = random.randint(start, end)
    blue = random.randint(start, end)
    if opacity is None:
        return (red, green, blue)
    return (red, green, blue, opacity)



import boto3
import asyncio
import aioboto3

KeySchema = {
    'AttributeName': '',
    'KeyType': ''
}

AttributeDefinitions = {
    'AttributeName': '',
    'AttributeType': ''
}

ProvisionedThroughput = {
    'ReadCapacityUnits': 10,
    'WriteCapacityUnits': 10
}


class DynamoDBHelper:
    def __init__(self, key_id, access_key, region='cn-north-1'):
        self.aws_access_key_id = key_id
        self.aws_secret_access_key = access_key

        self.db = boto3.resource(
            'dynamodb',
            region_name=region,
            aws_access_key_id=key_id,
            aws_secret_access_key=access_key)

    def create_table(
            self,
            table_name,
            hash_key_name,
            hash_key_type,
            range_key_name=None,
            range_key_type=None,
            read_capacity_units=10,
            write_capacity_units=10):
        """
        创建dynamodb表(支持简单主键:hash_key和复合主键:hash_key&range_key)
        :param hash_key_name:Hash主键名称
        :param hash_key_type:Hash主键类型 Valid Values: S | N | B
                                S - the attribute is of type String
                                N - the attribute is of type Number
                                B - the attribute is of type Binary
        :param range_key_name:Range主键名称
        :param range_key_type:Range主键类型 Valid Values: S | N | B
        :param read_capacity_units:每秒需对此表执行的读取次数。
        :param write_capacity_units:每秒需对此表执行的写入次数
        :return:
        """
        key_schema = []
        attribute_definitions = []
        KeySchema['AttributeName'] = hash_key_name
        KeySchema['KeyType'] = 'HASH'
        AttributeDefinitions['AttributeName'] = hash_key_name
        AttributeDefinitions['AttributeType'] = hash_key_type
        attribute_definitions.append(AttributeDefinitions)
        key_schema.append(KeySchema)
        if range_key_name:
            KeySchema['AttributeName'] = range_key_name
            KeySchema['KeyType'] = 'RANGE'
            AttributeDefinitions['AttributeName'] = range_key_name
            AttributeDefinitions['AttributeType'] = range_key_type
            attribute_definitions.append(AttributeDefinitions)
            key_schema.append(KeySchema)
        ProvisionedThroughput['ReadCapacityUnits'] = read_capacity_units
        ProvisionedThroughput['WriteCapacityUnits'] = write_capacity_units
        self.db.create_table(
            TableName=table_name,
            KeySchema=key_schema,
            AttributeDefinitions=attribute_definitions,
            ProvisionedThroughput=ProvisionedThroughput
        )
        return True

    def del_table(self, table_name):
        self.db.meta.client.delete_table(TableName=table_name)
        return True

    def put_item(self, table_name, item):
        """
        插入单条数据
        :param item:必须包含主键，key与表对应，格式正确的json
        """
        table = self.db.Table(table_name)
        table.put_item(Item=item)
        return True

    def batch_write_item(self, table_name, items):
        """
        批量插入item
        """
        table = self.db.Table(table_name)
        while len(items) > 0:
            item_list = items[0:25]
            with table.batch_writer() as batch:
                for item in item_list:
                    batch.put_item(item)
            items = items[25:]
        return True

    def del_item(self, table_name, key):
        """
        删除数据
        :param key: 格式正确的json
        """
        table = self.db.Table(table_name)
        table.delete_item(Key=key)
        return True

    async def async_get_item(
            self,
            table_name,
            key,
            consistent_read=False):
        """
        使用项目的主键读取项目
         :type key: dict
         :param key: {
                        "hash_key":"key_name",
                        "range_key":"key_name"
                     }
        """
        async with aioboto3.resource(
                'dynamodb',
                region_name='cn-north-1',
                aws_secret_access_key=self.aws_secret_access_key,
                aws_access_key_id=self.aws_access_key_id
        ) as resource:
            table = resource.Table(table_name)
            response = await asyncio.get_event_loop().create_task(
                table.get_item(
                    Key=key,
                    ConsistentRead=consistent_read)
            )
        if 'Item' in response:
            item = response['Item']
            return item
        return None

    async def async_batch_get_item(self,
                                   table_name,
                                   keys,
                                   consistent_read=False):
        """
        批量获取item
        """
        requestItems = {}
        itemlist = []
        async with aioboto3.resource(
                'dynamodb',
                region_name='cn-north-1',
                aws_secret_access_key=self.aws_secret_access_key,
                aws_access_key_id=self.aws_access_key_id
        ) as resource:
            while len(keys) > 0:
                key_list = keys[0:100]
                requestItems[table_name] = {
                    'Keys': key_list
                }
                response = await asyncio.get_event_loop().create_task(
                    resource.meta.client.batch_get_item(
                        RequestItems=requestItems))
                itemlist.extend(response['Responses'][table_name])
                keys = keys[100:]
        return itemlist



from PIL import Image, ImageOps
from io import BytesIO
from utilib.core.http import async_fetch_url
import qrcode
import six

NUL = b'\x00'
EOT = b'\x04'
ENQ = b'\x05'
DLE = b'\x10'
DC4 = b'\x14'
CAN = b'\x18'
ESC = b'\x1b'
FS = b'\x1c'
GS = b'\x1d'
NEWLINE = b'\x1b\x4a\x30'


def _generate_qrcode_image(text, **kwargs):
    _image = qrcode.make(
        text,
        error_correction=qrcode.ERROR_CORRECT_H,
        **kwargs)
    return _image


def generate_qrcode_image(text, **kwargs):

    _image = qrcode.make(
        text,
        error_correction=qrcode.ERROR_CORRECT_H,
        **kwargs)
    size = 384, int(_image.size[1] * 384 / _image.size[0] + .5)
    return _image.resize(size).convert('1')


def get_image_content(_image):
    if isinstance(_image, bytes):
        _c = BytesIO()
        _c.write(_image)
        _image = Image.open(_c).convert('RGBA')
    elif isinstance(_image, BytesIO):
        _image = Image.open(_image).convert('RGBA')
    _bg = Image.new("RGB", _image.size, (255, 255, 255))
    _bg.paste(_image, (0, 0), _image)
    _content = BytesIO()
    _bg.save(_content, format='JPEG')
    return _content.getvalue()


class ImageHelper:

    @staticmethod
    def _int_low_high(inp_number, out_bytes):
        """ Generate multiple bytes for a number: In lower and higher parts,
        or more parts as needed.

        :param inp_number: Input number
        :param out_bytes: The number of bytes to output (1 - 4).
        """
        max_input = (256 << (out_bytes * 8) - 1)
        if not 1 <= out_bytes <= 4:
            raise ValueError('Can only output 1-4 bytes')
        if not 0 <= inp_number <= max_input:
            raise ValueError(
                'Number too large. Can only output up to '
                '{0} in {1} bytes'.format(max_input, out_bytes))
        outp = b''
        for _ in range(0, out_bytes):
            outp += six.int2byte(inp_number % 256)
            inp_number //= 256
        return outp

    @staticmethod
    def image2bin_v2(_image, position=0):
        header = b'\x1b' + b'\x09' + b'\x1b' + b'\xfe' + bytes.fromhex(
            '%02d' % position) + ImageHelper._int_low_high(
            int(_image.width / 8),
            2) + ImageHelper._int_low_high(
            _image.height, 2)
        return header + _image.tobytes() + b'\x1b' + b'\x15'

    @staticmethod
    def image2bin(_image):
        density_byte = 0
        header = GS + b"v0" + six.int2byte(
            density_byte) + ImageHelper._int_low_high(
            int(_image.width / 8),
            2) + ImageHelper._int_low_high(
            _image.height, 2)
        return header + _image.tobytes() + b'\n' * 3

    @staticmethod
    def get_hex_from_bin(bin_content):
        content = bin_content
        result = ''
        count = 0
        while 1:
            if count > len(content) - 1:
                break
            c = content[count]
            count += 1
            result += hex(ord(c)).upper()[2:] + ' ' if not len(
                hex(ord(c)).upper()[2:]) == 1 else '0' + hex(
                ord(c)).upper()[2:] + ' '
        return result

    @staticmethod
    def get_receipt_image(image):
        image = ImageOps.invert(image.convert('L'))
        size = 384, int(image.size[1] * 384 / image.size[0] + .5)
        return image.resize(size).convert('1')

    @staticmethod
    async def get_template_demo(content):
        image = Image.open(content)
        image = ImageHelper.get_receipt_image(image)
        template_content = await async_fetch_url(
            'https://s3.cn-north-1.amazonaws.com.cn'
            '/ad-dist/template.png')
        template_content = template_content.body
        _template = BytesIO()
        _template.write(template_content)
        template_image = Image.open(_template)
        template_image = ImageHelper.get_receipt_image(template_image)
        blank_image = Image.new('1',
                                (384, template_image.size[1] + image.size[1]))
        blank_image.paste(template_image, (0, 0))
        blank_image.paste(image, (0, template_image.size[1]))
        return blank_image



import tornado.httpclient
import urllib


async def async_fetch_url(
        url, method='GET', headers={}, raise_error=False, **params):
    if method not in ('GET', 'POST', 'PATCH', 'PUT', 'DELETE'):
        return None
    request = None
    headers.update({'Content-Type': 'application/x-www-form-urlencoded'})
    if method == 'GET':
        request = tornado.httpclient.HTTPRequest(
            str.format('{url}?{params}',
                       url=url,
                       headers=headers,
                       params=urllib.parse.urlencode(params)))
    else:
        request = tornado.httpclient.HTTPRequest(
            str.format('{url}', url=url),
            headers=headers,
            body=urllib.parse.urlencode(params),
            method=method)
    client = tornado.httpclient.AsyncHTTPClient()
    response = await client.fetch(request, raise_error=raise_error)
    return response


def fetch_url(url, method='GET', headers={}, **params):
    if method not in ('GET', 'POST', 'PATCH', 'PUT', 'DELETE'):
        return None
    request = None
    headers.update({'Content-Type': 'application/x-www-form-urlencoded'})
    if method == 'GET':
        request = tornado.httpclient.HTTPRequest(
            str.format('{url}?{params}',
                       url=url,
                       headers=headers,
                       params=urllib.parse.urlencode(params)))
    else:
        request = tornado.httpclient.HTTPRequest(
            str.format('{url}', url=url),
            headers=headers,
            body=urllib.parse.urlencode(
                params) if method is not 'DELETE' else None,
            method=method)
    client = tornado.httpclient.HTTPClient()
    response = client.fetch(request, raise_error=False)
    return response



from utilib.core import http
import math
import json

x_pi = 3.14159265358979324 * 3000.0 / 180.0


def convert_gcj02_to_bd09(lat, lng):
    x = lng
    y = lat
    z = math.sqrt(x * x + y * y) + 0.00002 * math.sin(y * x_pi)
    theta = math.atan2(y, x) + 0.000003 * math.cos(x * x_pi)
    lat = z * math.sin(theta) + 0.006
    lng = z * math.cos(theta) + 0.0065
    return lat, lng


def convert_bd09_to_gcj02(lat, lng):
    x = lng - 0.0065
    y = lat - 0.006
    z = math.sqrt(x * x + y * y) - 0.00002 * math.sin(y * x_pi)
    theta = math.atan2(y, x) - 0.000003 * math.cos(x * x_pi)
    lat = z * math.sin(theta)
    lng = z * math.cos(theta)
    return lat, lng


async def get_district_by_adcode(key, adcode):
    _r = await http.async_fetch_url(
        'http://restapi.amap.com/v3/config/district',
        'GET',
        {},
        True,
        key=key,
        keywords=adcode
        )
    return json.loads(_r.body)



import os
import hashlib
import base64


def gen_salt(size=16):
    return os.urandom(size).hex()


def gen_password(password, iterations=24000):
    _salt = gen_salt()
    return base64.b64encode(
        hashlib.pbkdf2_hmac('sha256',
                            password.encode('utf-8'),
                            _salt.encode('utf-8'),
                            iterations)).decode(), _salt


def get_password(password, salt, iterations=24000):
    return base64.b64encode(
        hashlib.pbkdf2_hmac('sha256',
                            password.encode('utf-8'),
                            salt.encode('utf-8'),
                            iterations)).decode()



from io import BytesIO
import aiobotocore
import mimetypes
import asyncio
import boto3


class S3Helper:

    def __init__(self, host, access_key, key_id, region='cn-north-1'):
        self.host = host
        self.aws_secret_access_key = access_key
        self.aws_access_key_id = key_id

        self.session = boto3.Session(
            aws_access_key_id=key_id,
            aws_secret_access_key=access_key,
            region_name=region
        )

    async def async_upload(self, bucket_name, filename, content):
        session = aiobotocore.get_session()
        content_type = mimetypes.guess_type(filename)[0]
        async with session.create_client(
            's3',
            region_name='cn-north-1',
            aws_secret_access_key=self.aws_secret_access_key,
            aws_access_key_id=self.aws_access_key_id
        ) as client:
            if content_type:
                await asyncio.get_event_loop().create_task(
                    client.put_object(
                        Bucket=bucket_name,
                        Key=filename,
                        Body=content,
                        ACL='public-read',
                        ContentType=content_type))
            else:
                await asyncio.get_event_loop().create_task(
                    client.put_object(
                        Bucket=bucket_name,
                        Key=filename,
                        Body=content,
                        ACL='public-read'))
            return 'https://%s/%s/%s' % (
                self.host,
                bucket_name,
                filename)

    async def async_download_fileobj(self, bucket_name, filename):
        session = aiobotocore.get_session()
        out = BytesIO()
        async with session.create_client(
            's3',
            region_name='cn-north-1',
            aws_secret_access_key=self.aws_secret_access_key,
            aws_access_key_id=self.aws_access_key_id
        ) as client:
            _response = await client.get_object(
                Bucket=bucket_name, Key=filename)
            async with _response['Body'] as stream:
                out.write(await stream.read())
                return out

    def upload(self, bucket_name, filename, content):
        content_type = mimetypes.guess_type(filename)[0]

        s3 = self.session.resource('s3')

        if content_type:
            s3.Bucket(bucket_name).put_object(
                Key=filename,
                Body=content,
                ContentType=content_type
                )
        else:
            s3.Bucket(bucket_name).put_object(
                Key=filename,
                Body=content
                )
        s3.meta.client._endpoint.http_session.close()
        return 'https://%s/%s/%s' % (self.host, bucket_name, filename)

    def download_fileobj(self, bucket_name, filename):
        out = BytesIO()
        s3 = self.session.resource('s3')
        s3.Bucket(bucket_name).download_fileobj(Key=filename, Fileobj=out)
        s3.meta.client._endpoint.http_session.close()
        return out



from dateutil import parser, tz
import datetime


_tzinfo = tz.gettz('Asia/Shanghai')


def to_str(datetime, format='%Y-%m-%d %H:%M:%S'):
    return datetime.strftime(format)


def cst(datetime, format='%Y-%m-%d %H:%M'):
    return datetime.astimezone(_tzinfo).strftime(format)


def utc(datetime, format='%Y-%m-%d %H:%M:%S%z'):
    return datetime.astimezone(tz.gettz('UTC')).strftime(format)


def now(_tz=tz.gettz('UTC')):
    return datetime.datetime.utcnow().replace(
        tzinfo=tz.gettz('UTC')).astimezone(_tz)


def today(_tz=tz.gettz('Asia/Shanghai')):
    _today = now(_tz=_tz)
    return datetime.datetime(
        _today.year, _today.month,
        _today.day, tzinfo=_tz)


def timeline(start, end):
    if not isinstance(start, datetime.datetime):
        start = parser.parse(start).astimezone(tz.gettz('Asia/Shanghai'))
    if not isinstance(end, datetime.datetime):
        end = parser.parse(end).astimezone(tz.gettz('Asia/Shanghai'))
    start = datetime.datetime(
        start.year, start.month,
        start.day, start.hour, tzinfo=tz.gettz('Asia/Shanghai'))
    end = datetime.datetime(
        end.year, end.month,
        end.day, end.hour,
        tzinfo=tz.gettz('Asia/Shanghai')) + datetime.timedelta(hours=1)
    return [start + datetime.timedelta(hours=_i) for _i in range(
        int((end - start).total_seconds() / 3600 - 1))]
